"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Driver = void 0;
const tslib_1 = require("tslib");
const debug_1 = tslib_1.__importDefault(require("debug"));
const orm_1 = require("./orm");
const debug = (0, debug_1.default)('bleco:query:driver');
class Driver {
    constructor(dataSource, options = {}) {
        this.dataSource = dataSource;
        this.options = options;
        this.orm = new orm_1.DefaultOrm(dataSource);
        this.init();
    }
    init() {
        // do nothing
    }
    applySortPolicy(model, filter) {
        const sortPolicy = this.getDefaultIdSortPolicy(model);
        const sortById = (() => {
            switch (sortPolicy) {
                case 'numericIdOnly':
                    return this.hasOnlyNumericIds(model);
                case false:
                    return false;
                default:
                    return true;
            }
        })();
        debug(model.modelName, 'sort policy:', sortPolicy, sortById);
        if (sortById && !filter.order) {
            const idNames = this.orm.idNames(model.modelName);
            if (idNames === null || idNames === void 0 ? void 0 : idNames.length) {
                filter.order = idNames;
            }
        }
    }
    getDefaultIdSortPolicy(model) {
        const definition = model.definition;
        if (Object.hasOwn(definition.settings, 'defaultIdSort')) {
            return definition.settings.defaultIdSort;
        }
        return true;
    }
    hasOnlyNumericIds(model) {
        const cols = model.definition.properties;
        const idNames = this.orm.idNames(model.modelName);
        const numericIds = idNames.filter(idName => cols[idName].type === Number);
        return numericIds.length === idNames.length;
    }
}
exports.Driver = Driver;
//# sourceMappingURL=driver.js.map