"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.KnexSupportedClientRegexps = exports.KnexSupportedClients = void 0;
exports.createKnex = createKnex;
const tslib_1 = require("tslib");
const string_1 = tslib_1.__importDefault(require("tily/is/string"));
exports.KnexSupportedClients = {
    MsSQL: 'mssql',
    MySQL: 'mysql',
    MySQL2: 'mysql2',
    Oracle: 'oracledb',
    PostgreSQL: 'pg',
    PgNative: 'pgnative',
    Redshift: 'pg-redshift',
    SQLite: 'sqlite3',
    CockroachDB: 'cockroachdb',
    BetterSQLite3: 'better-sqlite3',
};
exports.KnexSupportedClientRegexps = {
    [exports.KnexSupportedClients.MsSQL]: /^mssql|ms$/i,
    [exports.KnexSupportedClients.MySQL]: /^mysql|maria|mariadb$/i,
    [exports.KnexSupportedClients.MySQL2]: /^mysql2$/i,
    [exports.KnexSupportedClients.Oracle]: /^oracledb$/i,
    [exports.KnexSupportedClients.PostgreSQL]: /^pg|postgres|postgresql$/i,
    [exports.KnexSupportedClients.PgNative]: /^pgnative$/i,
    [exports.KnexSupportedClients.Redshift]: /^redshift$/i,
    [exports.KnexSupportedClients.SQLite]: /^sqlite3?$/i,
    [exports.KnexSupportedClients.CockroachDB]: /^cockroachdb$/i,
    [exports.KnexSupportedClients.BetterSQLite3]: /^better-sqlite3$/i,
};
// eslint-disable-next-line @typescript-eslint/no-explicit-any
function createKnex(ds, options) {
    var _a, _b;
    const dialect = (_a = options === null || options === void 0 ? void 0 : options.client) !== null && _a !== void 0 ? _a : (_b = ds.connector) === null || _b === void 0 ? void 0 : _b.name;
    const client = resolveKnexClientWithDialect(dialect);
    if (!client) {
        throw new Error(`Unsupported dialect: ${dialect}`);
    }
    const config = {
        client,
        wrapIdentifier: (value, origImpl, queryContext) => {
            if (queryContext === null || queryContext === void 0 ? void 0 : queryContext.skipEscape) {
                return value;
            }
            return origImpl(value);
        },
    };
    if ((0, string_1.default)(client) && client.match(/sqlite/i)) {
        // suppress knex sqlite3 useNullAsDefault warning
        config.useNullAsDefault = true;
    }
    return require('knex')(config);
}
function resolveKnexClientWithDialect(dialect) {
    if (typeof dialect === 'string') {
        for (const client in exports.KnexSupportedClientRegexps) {
            if (exports.KnexSupportedClientRegexps[client].test(dialect)) {
                return client;
            }
        }
    }
    else if (typeof dialect === 'function') {
        return dialect;
    }
    return null;
}
//# sourceMappingURL=knex.js.map