"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ColumnsResolver = void 0;
const tslib_1 = require("tslib");
/* eslint-disable @typescript-eslint/no-floating-promises */
const filter_1 = require("@loopback/filter");
const empty_1 = tslib_1.__importDefault(require("tily/is/empty"));
const object_1 = tslib_1.__importDefault(require("tily/is/object"));
const clause_1 = require("../clause");
class ColumnsResolver extends clause_1.ClauseResolver {
    resolve(qb, filter) {
        const columns = this.buildColumnNames((0, filter_1.isFilter)(filter) ? filter.fields : filter, true);
        if (columns) {
            qb.columns(columns);
        }
    }
    buildColumnNames(fields, withTable = false) {
        const props = this.entityClass.definition.properties;
        if ((0, empty_1.default)(props)) {
            return;
        }
        let keys = Object.keys(props);
        if (fields && !(0, empty_1.default)(fields)) {
            if (Array.isArray(fields)) {
                keys = fields.filter(f => props[f]);
            }
            else if ((0, object_1.default)(fields)) {
                const includes = [];
                const excludes = [];
                for (const key of keys) {
                    if (fields[key]) {
                        includes.push(key);
                    }
                    else if (key in fields && !fields[key]) {
                        excludes.push(key);
                    }
                }
                if (includes.length > 0) {
                    keys = includes;
                }
                else if (excludes.length > 0) {
                    keys = keys.filter(k => !excludes.includes(k));
                }
            }
        }
        return keys.map(key => this.columnEscaped(key, withTable));
    }
}
exports.ColumnsResolver = ColumnsResolver;
//# sourceMappingURL=columns.js.map