"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StatementResolvers = exports.StatementResolver = void 0;
const resolvers_1 = require("./resolvers");
class StatementResolver {
    constructor(model, orm) {
        this.model = model;
        this.orm = orm;
        this.columns = new resolvers_1.ColumnsResolver(model, this.orm);
        this.join = new resolvers_1.JoinResolver(model, this.orm);
        this.where = new resolvers_1.WhereResolver(model, this.orm);
        this.order = new resolvers_1.OrderResolver(model, this.orm);
    }
    resolveColumns(qb, filter) {
        this.columns.resolve(qb, filter);
    }
    resolveJoin(qb, filter, session) {
        this.join.resolve(qb, filter, session);
    }
    resolveWhere(qb, filter, session) {
        this.where.resolve(qb, filter, session);
    }
    resolveOrder(qb, filter, session) {
        this.order.resolve(qb, filter, session);
    }
}
exports.StatementResolver = StatementResolver;
class StatementResolvers {
    constructor(orm) {
        this.orm = orm;
        this.items = new Map();
    }
    get(model) {
        let answer = this.items.get(model.modelName);
        if (!answer) {
            answer = new StatementResolver(model, this.orm);
            this.items.set(model.modelName, answer);
        }
        return answer;
    }
}
exports.StatementResolvers = StatementResolvers;
//# sourceMappingURL=statement.js.map