import { MixinTarget } from '@loopback/core';
import { Where } from '@loopback/filter';
import { Entity, EntityCrudRepository } from '@loopback/repository';
import { QueryFilter, QueryWhere } from '../filter';
import { Query } from '../query';
export interface QueryEnhancedRepository<M extends Entity, Relations extends object = {}> {
    readonly query?: Query<M, Relations> | null;
}
export interface QueryMixinOptions {
    overrideCruds?: boolean;
}
/**
 * A mixin to add query support to a repository.
 *
 * @param superClass - Base class
 * @param mixinOptions - Mixin options
 */
export declare function QueryRepositoryMixin<M extends Entity, ID, Relations extends object, R extends MixinTarget<EntityCrudRepository<M, ID, Relations>>>(superClass: R, mixinOptions?: boolean | QueryMixinOptions): {
    new (...args: any[]): {
        __query__?: Query<M, Relations> | null;
        readonly query: Query<M, Relations> | null | undefined;
        find(filter?: QueryFilter<M>, options?: object): Promise<(M & Relations)[]>;
        findOne(filter?: QueryFilter<M>, options?: object): Promise<(M & Relations) | null>;
        count(where?: QueryWhere<M>, options?: object): Promise<{
            count: number;
        }>;
        entityClass: typeof Entity & {
            prototype: M;
        };
        inclusionResolvers: Map<string, import("@loopback/repository").InclusionResolver<M, Entity>>;
        save: (entity: import("@loopback/repository").DataObject<M>, options?: import("@loopback/repository").Options) => Promise<M>;
        update: (entity: import("@loopback/repository").DataObject<M>, options?: import("@loopback/repository").Options) => Promise<void>;
        delete: (entity: import("@loopback/repository").DataObject<M>, options?: import("@loopback/repository").Options) => Promise<void>;
        findById: (id: ID, filter?: import("@loopback/filter").FilterExcludingWhere<M> | undefined, options?: import("@loopback/repository").Options) => Promise<M & Relations>;
        updateById: (id: ID, data: import("@loopback/repository").DataObject<M>, options?: import("@loopback/repository").Options) => Promise<void>;
        replaceById: (id: ID, data: import("@loopback/repository").DataObject<M>, options?: import("@loopback/repository").Options) => Promise<void>;
        deleteById: (id: ID, options?: import("@loopback/repository").Options) => Promise<void>;
        exists: (id: ID, options?: import("@loopback/repository").Options) => Promise<boolean>;
        execute: (command: import("@loopback/repository").Command, parameters: import("@loopback/repository").NamedParameters | import("@loopback/repository").PositionalParameters, options?: import("@loopback/repository").Options) => Promise<import("@loopback/repository").AnyObject>;
        create: (dataObject: import("@loopback/repository").DataObject<M>, options?: import("@loopback/repository").Options) => Promise<M>;
        createAll: (dataObjects: import("@loopback/repository").DataObject<M>[], options?: import("@loopback/repository").Options) => Promise<M[]>;
        updateAll: (dataObject: import("@loopback/repository").DataObject<M>, where?: Where<M> | undefined, options?: import("@loopback/repository").Options) => Promise<import("@loopback/repository").Count>;
        deleteAll: (where?: Where<M> | undefined, options?: import("@loopback/repository").Options) => Promise<import("@loopback/repository").Count>;
    };
} & R;
