"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.QueryRepositoryMixin = QueryRepositoryMixin;
const query_1 = require("../query");
const debug = require('debug')('bleco:query:query-mixin');
/**
 * A mixin to add query support to a repository.
 *
 * @param superClass - Base class
 * @param mixinOptions - Mixin options
 */
function QueryRepositoryMixin(superClass, mixinOptions = {}) {
    const opts = typeof mixinOptions === 'boolean' ? { overrideCruds: mixinOptions } : mixinOptions;
    const { overrideCruds = true } = opts !== null && opts !== void 0 ? opts : {};
    return class extends superClass {
        get query() {
            if (this.__query__ === undefined && this.dataSource) {
                try {
                    this.__query__ = new query_1.DefaultQuery(this);
                }
                catch (e) {
                    this.__query__ = null;
                }
            }
            return this.__query__;
        }
        // @ts-ignore
        async find(filter, options) {
            if (overrideCruds && this.query) {
                debug(`${this.constructor.name} find() -> query.find()`);
                return this.query.find(filter, options);
            }
            debug(`${this.constructor.name} find() -> super.find()`);
            return super.find(filter, options);
        }
        async findOne(filter, options) {
            if (overrideCruds && this.query) {
                debug(`${this.constructor.name} findOne() -> query.findOne()`);
                return this.query.findOne(filter, options);
            }
            // @ts-ignore
            if (!super.findOne) {
                throw new Error('findOne is not implemented in this repository');
            }
            debug(`${this.constructor.name} findOne() -> super.findOne()`);
            // @ts-ignore
            return super.findOne(filter, options);
        }
        // @ts-ignore
        async count(where, options) {
            if (overrideCruds && this.query) {
                debug(`${this.constructor.name} count() -> query.count()`);
                return this.query.count(where, options);
            }
            debug(`${this.constructor.name} count() -> super.count()`);
            return super.count(where, options);
        }
    };
}
//# sourceMappingURL=query.mixin.js.map