"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DefaultOrm = void 0;
const repository_1 = require("@loopback/repository");
const assert_1 = require("tily/assert");
class DefaultOrm {
    constructor(ds) {
        this.ds = ds;
    }
    get connector() {
        var _a;
        return (_a = this.ds) === null || _a === void 0 ? void 0 : _a.connector;
    }
    getModelDefinition(modelName) {
        if (this.connector) {
            return this.connector.getModelDefinition(modelName);
        }
        throw new Error(`"connector" is required to get model definition for ${modelName}`);
    }
    idNames(model) {
        if (this.connector) {
            this.ensureModelHasBeenDefined(model);
            return this.connector.idNames(model);
        }
        return [];
    }
    escapeName(name) {
        if (this.connector) {
            return this.connector.escapeName(name);
        }
        return name;
    }
    column(model, prop) {
        if (this.connector) {
            this.ensureModelHasBeenDefined(model);
            return this.connector.column(model, prop);
        }
        return prop;
    }
    columnEscaped(model, property, withTable = false, prefix = '') {
        if (this.connector) {
            this.ensureModelHasBeenDefined(model);
            const column = this.connector.columnEscaped(model, property);
            if (withTable) {
                return this.connector.escapeName(prefix + this.connector.table(model)) + '.' + column;
            }
            return column;
        }
        return withTable ? `${prefix}${model}.${property}` : property;
    }
    table(model) {
        if (this.connector) {
            this.ensureModelHasBeenDefined(model);
            return this.connector.table(model);
        }
        return model;
    }
    tableEscaped(model) {
        if (this.connector) {
            this.ensureModelHasBeenDefined(model);
            return this.connector.tableEscaped(model);
        }
        return model;
    }
    buildColumnNames(model, filter) {
        if (this.connector) {
            this.ensureModelHasBeenDefined(model);
            const c = this.connector.buildColumnNames(model, filter);
            return Array.isArray(c) ? c : c.split(',').map(s => s.trim());
        }
        return [];
    }
    fromRow(model, rowData) {
        if (this.connector) {
            this.ensureModelHasBeenDefined(model);
            return this.connector.fromRow(model, rowData);
        }
        return rowData;
    }
    async execute(...args) {
        (0, assert_1.assert)(this.ds, `Orm is missing a datasource to execute the command.`);
        return (0, repository_1.ensurePromise)(this.ds.execute(...args));
    }
    ensureModelHasBeenDefined(model) {
        if (!this.connector) {
            throw new Error(`"connector" is required to check model for ${model}`);
        }
        if (!this.connector.getModelDefinition(model)) {
            throw new Error(`Model "${model}" is not defined in current connector "${this.connector.name}".`);
        }
    }
}
exports.DefaultOrm = DefaultOrm;
//# sourceMappingURL=orm.js.map