"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DefaultQuery = exports.QueryProperties = exports.QueryMethods = void 0;
exports.isQuery = isQuery;
/* eslint-disable @typescript-eslint/no-explicit-any */
const repository_1 = require("@loopback/repository");
const assert_1 = require("tily/assert");
const driver_loader_1 = require("./driver.loader");
exports.QueryMethods = ['find', 'findOne', 'count'];
exports.QueryProperties = ['entityClass', ...exports.QueryMethods];
class DefaultQuery {
    constructor(entityClassOrRepo, ds) {
        if (typeof entityClassOrRepo === 'function') {
            this.entityClass = entityClassOrRepo;
            this.dataSource = ds;
        }
        else {
            this.repo = entityClassOrRepo;
            this.entityClass = entityClassOrRepo.entityClass;
            this.dataSource = ds !== null && ds !== void 0 ? ds : entityClassOrRepo.dataSource;
        }
        (0, assert_1.assert)(this.dataSource, 'dataSource is required');
        this.driver = (0, driver_loader_1.loadDriver)(this.dataSource);
    }
    async find(filter, options) {
        filter = filter !== null && filter !== void 0 ? filter : {};
        const data = await this.driver.find(this.entityClass, filter, options);
        const modelClass = this.dataSource.getModel(this.entityClass.definition.name);
        if (!modelClass) {
            throw new Error(`Model class ${this.entityClass.definition.name} not found`);
        }
        const objects = data
            .map(m => new modelClass(m, {
            fields: filter === null || filter === void 0 ? void 0 : filter.fields,
            applySetters: false,
            persisted: true,
            // see https://github.com/strongloop/loopback-datasource-juggler/issues/1692
            applyDefaultValues: false,
        }))
            .map(m => m.toObject());
        const entities = this.toEntities(objects);
        if (this.repo) {
            return (0, repository_1.includeRelatedModels)(this.repo, entities, filter.include, options);
        }
        return entities;
    }
    async findOne(filter, options) {
        var _a;
        filter = filter !== null && filter !== void 0 ? filter : {};
        filter.limit = 1;
        const result = await this.find(filter, options);
        return (_a = result[0]) !== null && _a !== void 0 ? _a : null;
    }
    async count(where, options) {
        return this.driver.count(this.entityClass, where, options);
    }
    toEntity(model) {
        return new this.entityClass(model);
    }
    toEntities(models) {
        return models.map(m => this.toEntity(m));
    }
}
exports.DefaultQuery = DefaultQuery;
function isQuery(obj) {
    return (obj &&
        (obj instanceof DefaultQuery || (exports.QueryProperties.every(p => p in obj) && !(obj instanceof repository_1.DefaultCrudRepository))));
}
//# sourceMappingURL=query.js.map